/*
   mp_fast_alert.c

   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: libmp_fast_alert.c,v 1.2 2004/12/11 21:53:03 jleplast Exp $ */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <assert.h>
/* FARM9_BSD_PORT Added to work under FreeBSD */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include "mp_export.h"

/* declarations
 */

/* to remember the handler that our host will pass to us
   during initialization */
serv_t  *OS;

/* Our host expects these functions */
mp_out_init_t mp_out_init;
mp_out_alert_t mp_out_alert;
mp_out_fini_t mp_out_fini;

/*  P R O T O T Y P E S  ************************************************/
static void ts_print(FILE *, const struct timeval *tv);

/* Global variables */
static FILE *OUT = NULL;

/* implementations */

/* this function (if available) gets executed automatically
   during initialization. Unfortunately, this phat copyright
   message won't be seen in the daemon mode. */
void _init()
{
  printf(
	 "---------------------------------------------------\n"
	 "        Fast Alert output module for mudpit.\n"
	 "   (c) 2002 G Savchuk, Fidelis Security Systems.\n"
	 "---------------------------------------------------\n"
	 );
  fflush(stdout);
}

int mp_out_init(char *args, serv_t *os)
{
  assert(OS = os);

  if(args) {
    if(!(OUT = fopen(args, "w")))
      OS->fatal("open \"%s\": %s\n", args, strerror(errno));
  }
  if(!OUT) OUT = stdout;

  return 1;
}

static void ts_print(FILE *f, const struct timeval *tv)
{
  register int s;
  time_t Time;
  struct tm *tm;
  int thiszone = 0;

  assert(f);
  assert(tv);

  s = (tv->tv_sec + thiszone) % 86400;
  Time = (tv->tv_sec + thiszone) - s;
  tm  = gmtime (&Time);
  (void)fprintf(f, "%02d/%02d/%04d %02d:%02d:%02d.%06u ",
	       tm->tm_mon+1, tm->tm_mday,
	       tm->tm_year+1900,
	       s / 3600, (s % 3600) / 60,
	       s % 60, (unsigned)tv->tv_usec);
}

int mp_out_alert(void *data)
{
  UnifiedAlertRecord *uar = (UnifiedAlertRecord *)data;
  class_t *ct = OS->get_class(uar->event.classification);
  sid_t *sid = OS->get_sid(uar->event.sig_generator, uar->event.sig_id);
  ref_t *ref = NULL;
  char s[256], d[256];

  int so = ntohl(uar->sip);
  int de = ntohl(uar->dip);

#define SIA struct in_addr
  strncpy(s, inet_ntoa((SIA)*((SIA *)&so)), sizeof(s));
  strncpy(d, inet_ntoa((SIA)*((SIA *)&de)), sizeof(d));
#undef SIA

  ts_print(OUT, &uar->ts);
  fprintf(OUT, "{%s} ", OS->proto_name(uar->protocol));

  if(uar->protocol == IPPROTO_TCP || uar->protocol == IPPROTO_UDP)
    fprintf(OUT, "%s:%d -> %s:%d\n", s, uar->sp, d, uar->dp);
  else
    fprintf(OUT, "%s -> %s\n", s, d);

  fprintf(OUT,
	  "[**] [%d:%d:%d] %s [**]\n"
	  "[Classification: %s] [Priority: %d]\n",
	  uar->event.sig_generator, uar->event.sig_id, uar->event.sig_rev,
	  sid ? sid->msg : "ALERT", ct ? ct->name : "Unknown",
	  uar->event.priority);


  for(ref = sid->ref; ref; ref = ref->next) {
    char buf[1024];
    fprintf(OUT, "[Xref => %s:%s (%s)]\n", ref->system, ref->id,
	    OS->get_htmlhdr(buf, sizeof(buf), ref->system, ref->id));
  }

  fprintf(OUT, "------------------------------------------------------------------------\n");

  return 1;
}

void mp_out_fini()
{
  if(OUT)
    fclose(OUT);
}

